package aacs.main;

import aacs.shared.Utils;
import aacs.decoder.DecryptEVOB;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Date;

public class BackupHDDVD {

    public static void main(String argv[]){
    System.out.println("Backup HD-DVD V0.999 Starting");
    if (argv.length!=2){
        System.out.println("Usage: BackupHDDVD SourceDrive Destination_directory");
        System.out.println("Example: BackupHDDVD f: e:\\movie\\mymovie");
        System.exit(0);
        }
     System.out.println("Testing source");
     File source = new File(argv[0]+File.separator+"aacs");
     if (!source.exists()){
         System.out.println("Could not find HD-DVD source "+argv[0]);
         System.exit(0);
         }
     File keyFile = new File(argv[0]+File.separator+"aacs"+File.separator+"VTKF000.AACS");
        if (!keyFile.exists()){
            System.out.println("Could not find keyFile "+keyFile.getAbsolutePath());
            System.exit(0);
        }

    File destination = new File(argv[1]);
        if (!destination.exists()||!destination.isDirectory()){
            System.out.println("Missing or invalid directory "+destination.getAbsolutePath());
            System.exit(0);
        }

    System.out.println("Found valid HD-DVD source.");

    byte[] keyFileHash = Utils.hashFile(keyFile);
    System.out.println("Look for this movie in my database");
    Properties TitleKeyDatabase = new Properties();
    try{
    TitleKeyDatabase.load(new FileInputStream("TKDB.cfg"));
    String keys= TitleKeyDatabase.getProperty(Utils.toHexString(keyFileHash));
    if (keys==null){
        System.out.println("Could not find this movie in my database. Please update your title key file");
        System.exit(0);
    }
    StringTokenizer st = new StringTokenizer(keys,"|");
    String title = st.nextToken();
    System.out.println("Found movie: "+title);
    Hashtable keySet=loadKeys(keys);
    System.out.println("Start backup on "+ new Date());
    System.out.println("Scaning video directory");
    File files[]= new File(argv[0]+File.separator+"HVDVD_TS").listFiles();
    for (int c=0;c<files.length;c++){
        if (files[c].isFile()){
            if(files[c].getName().endsWith(".EVO")){
                int keyNumber = getKeyFromEVOB(files[c]);
                if (keyNumber!=0){
                    if (keySet.get(keyNumber)!=null){
                        //
                        System.out.println("Processing "+files[c].getName()+" key = "+keyNumber);
                        DecryptEVOB.decryptEVOBFile(files[c],new File(argv[1]+File.separator+files[c].getName()) , keySet);
                    }else{
                        System.out.println("Could not find key "+keyNumber+" Skipping file "+files[c].getName());
                    }
                }else{
                    System.out.println("Skipping file "+files[c].getName());
                }
            }
        }
    }
    System.out.println("End backup on "+ new Date());
            }catch(Exception e){
                e.printStackTrace();
                System.out.println("Error loading Tilte key file TKDB.cfg. Aborting");
            }
    }

    public static Hashtable loadKeys(String keysInfo){
        Hashtable keyList = new Hashtable();
        StringTokenizer st = new StringTokenizer(keysInfo,"|");
        st.nextToken(); // readTitle
        while (st.hasMoreTokens()){
            String aKey = st.nextToken();
            int pos= aKey.indexOf("-");
            if (pos==-1){
                System.out.println("Format Error int title key file. Aborting ");
                System.exit(0);
            }
            Integer keyNumber = Integer.parseInt(aKey.substring(0,pos).trim());
            byte[] keyvalue= Utils.hexStringToByteArray(aKey.substring(pos+1));
            keyList.put(keyNumber,keyvalue);
        }
        return keyList;
    }

    public static int getKeyFromEVOB(File anEVOBFile){
        byte[] header= new byte[128];
        try{
            FileInputStream fis = new  FileInputStream(anEVOBFile);
            fis.read(header);
            return header[0x3e];
        }catch(Exception e){
            e.printStackTrace();
        }
        return 0;
    }

}
